from datetime import timedelta
import datetime
from duplocloud.client import DuploClient
from duplocloud.resource import DuploResource
from duplocloud.errors import DuploError
from duplocloud.commander import Command, Resource
import duplocloud.args as args

@Resource("tenant")
class DuploTenant(DuploResource):
  def __init__(self, duplo: DuploClient):
    super().__init__(duplo)
  
  @Command()
  def list(self):
    """Retrieve a list of all tenants in the Duplo system."""
    response = self.duplo.get("adminproxy/GetTenantNames")
    return response.json()

  @Command()
  def find(self, 
           name: args.NAME):
    """Find a tenant by name."""
    try:
      return [t for t in self.list() if t["AccountName"] == name][0]
    except IndexError:
      raise DuploError(f"Tenant '{name}' not found", 404)
  
  @Command()
  def create(self, 
             body: args.BODY):
    """Create a new tenant."""
    self.duplo.post("admin/AddTenant", body)
    return {
      "message": f"Tenant '{body['AccountName']}' created"
    }
    
  @Command()
  def shutdown(self, 
               name: args.NAME, 
               schedule: args.SCHEDULE=None):
    """Expire a tenant."""
    tenant = self.find(name)
    tenant_id = tenant["TenantId"]
    # if the schedule not specified then set the date 5 minute from now
    if schedule is None:
      now = datetime.datetime.now() + datetime.timedelta(minutes=5)
      schedule = now.strftime("%a, %d %b %Y %H:%M:%S GMT")
    else:
      interval, unit = int(schedule[:-1]), schedule[-1]
      # Calculate the timedelta based on the specified time interval
      if unit == 'm':
        delta = timedelta(minutes=interval)
      elif unit == 'h':
        delta = timedelta(hours=interval)
      elif unit == 'd':
        delta = timedelta(days=interval)
      else:
          raise ValueError("Invalid time unit specified. Please use 'm' for minutes, 'h' for hours, or 'd' for days.")

      current_time = datetime.datetime.utcnow()
      # Calculate the future time after adding the timedelta
      future_time = current_time + delta
      # Format the future time in the desired string format
      schedule = future_time.strftime("%a, %d %b %Y %H:%M:%S GMT")

    res = self.duplo.post("adminproxy/UpdateTenantCleanupTimers", {
      "TenantId": tenant_id,
      "PauseTime": schedule
    })

    if res.status_code == 200:
      return f"Tenant '{name}' will shutdown on {schedule}"
    else:
      raise DuploError(f"Failed to expire tenant '{name}'", res.status_code)

  @Command()
  def logging(self, 
              name: args.NAME, 
              enable: args.ENABLE=True):
    """Enable or disable tenant logging."""
    tenant = self.find(name)
    tenant_id = tenant["TenantId"]
    # add or update the tenant in the list of enabled tenants
    response = self.duplo.get("admin/GetLoggingEnabledTenants")
    log_tenants = response.json()
    for t in log_tenants:
      if t["TenantId"] == tenant_id:
        t["Enabled"] = enable
        break
    else:
      log_tenants.append({"TenantId": tenant_id, "Enabled": enable})
    print(log_tenants)
    # update the entire list
    res = self.duplo.post("admin/UpdateLoggingEnabledTenants", log_tenants)
    if res.status_code == 200:
      return f"Tenant '{name}' logging {enable}"
    else:
      raise DuploError(f"Failed to {'enable' if enable else 'disable'} tenant '{name}'", res.status_code)

  @Command()
  def billing(self,
           name: args.NAME):
    """Spend
    
    Get the spend for the tenant. 
    """
    tenant = self.find(name)
    tenant_id = tenant["TenantId"]
    response = self.duplo.get(f"v3/billing/subscriptions/{tenant_id}/aws/billing")
    return response.json()



from duplocloud.client import DuploClient
from duplocloud.resource import DuploTenantResource
from duplocloud.commander import Command, Resource
import duplocloud.args as args

@Resource("cronjob")
class DuploCronJob(DuploTenantResource):
  
  def __init__(self, duplo: DuploClient):
    super().__init__(duplo)
  
  @Command()
  def list(self):
    """Retrieve a list of all cronjob in a tenant."""
    tenant_id = self.tenant["TenantId"]
    response = self.duplo.get(f"v3/subscriptions/{tenant_id}/k8s/cronJob")
    return response.json()
  
  @Command()
  def find(self, 
           name: args.NAME):
    """Find a cronjob by name.
    
    Args:
      name (str): The name of the cronjob to find.
    Returns: 
      The cronjob object.
    Raises:
      DuploError: If the cronjob could not be found.
    """
    tenant_id = self.tenant["TenantId"]
    response = self.duplo.get(f"v3/subscriptions/{tenant_id}/k8s/cronJob/{name}")
    return response.json()

  @Command()
  def update_image(self, 
                   name: args.NAME, 
                   image: args.IMAGE):
    """Update the image of a cronjob.
    
    Args:
      name (str): The name of the cronjob to update.
      image (str): The new image to use for the cronjob.
    """
    tenant_id = self.tenant["TenantId"]
    cronjob = self.find(name)
    cronjob["spec"]["jobTemplate"]["spec"]["template"]["spec"]["containers"][0]["image"] = image
    self.duplo.put(f"v3/subscriptions/{tenant_id}/k8s/cronJob/{name}", cronjob)
    return {"message": f"Successfully updated image for cronjob '{name}'"}
  
  @Command()
  def update_schedule(self, 
                      name: args.NAME, 
                      cronschedule: args.CRONSCHEDULE):
    """Update the schedule of a cronjob.
    
    Args:
      name (str): The name of the cronjob to update.
      schedule (str): The new schedule to use for the cronjob.
    """
    tenant_id = self.tenant["TenantId"]
    cronjob = self.find(name)
    cronjob["spec"]["schedule"] = cronschedule
    self.duplo.put(f"v3/subscriptions/{tenant_id}/k8s/cronJob/{name}", cronjob)
    return {"message": f"Successfully updated cron-schedule for cronjob '{name}'"}
  
  



from duplocloud.client import DuploClient
from duplocloud.resource import DuploTenantResource
from duplocloud.errors import DuploError
from duplocloud.commander import Command, Resource
import duplocloud.args as args

@Resource("asg")
class DuploAsg(DuploTenantResource):
  
  def __init__(self, duplo: DuploClient):
    super().__init__(duplo)
  
  @Command()
  def list(self):
    """Retrieve a list of all services in a tenant."""
    tenant_id = self.tenant["TenantId"]
    response = self.duplo.get(f"subscriptions/{tenant_id}/GetTenantAsgProfiles")
    return response.json()

  @Command()
  def find(self, 
           name: args.NAME):
    """Find an asg by name.
    
    Args:
      name (str): The name of the asg to find.
    Returns: 
      The service object.
    Raises:
      DuploError: If the asg could not be found.
    """
    try:
      return [s for s in self.list() if s["FriendlyName"] == name][0]
    except IndexError:
      raise DuploError(f"ASG Profile '{name}' not found", 404)

  @Command()
  def scale(self,
            name: args.NAME,
            min: args.MIN=None,
            max: args.MAX=None):
    """Scale an ASG."""
    if not min and not max:
      raise DuploError("Must provide either min or max")
    tenant_id = self.tenant["TenantId"]
    asg = self.find(name)
    data = {
      "FriendlyName": name,
      "DesiredCapacity": asg.get("MinSize", None),
      "MinSize": asg.get("MinSize", None),# this really is a string unlike the other two? 
      "MaxSize": asg.get("MaxSize", None),
    }
    if min:
      data["MinSize"] = str(min)
    if max:
      data["MaxSize"] = max
    res = self.duplo.post(f"subscriptions/{tenant_id}/UpdateTenantAsgProfile", data)
    return {
      "message": f"Successfully updated asg '{name}'",
      "data": res.json()
    }
  


import os
import pytest

from duplocloud.errors import DuploError
from duplocloud.client import DuploClient

# current working directory as variable
cwd = os.getcwd()
host = "http://example.duplocloud.net"
cache_dir = f"{cwd}/.tmp/cache"

def test_new_config():
  c = DuploClient(host=host)
  assert c.host == host

def test_at_least_host():
  """No Host Gets Error"""
  duplo = DuploClient()
  with pytest.raises(DuploError) as e:
    duplo.token
    print(e)

def test_cache_dir():
  c = DuploClient(
    host=host,
    cache_dir=cache_dir)
  assert c.cache_dir == cache_dir
  random_data = {"foo": "bar"}
  cf = f"{cache_dir}/test.json"
  c.set_cached_item("test", random_data)
  assert os.path.exists(cf), f"Cache file {cf} not found"
  # now check if we can get the data back
  assert c.get_cached_item("test") == random_data, "Cached data does not match"
  # delete the cache file
  os.remove(cf)
  os.rmdir(cache_dir)



from duplocloud.client import DuploClient
from duplocloud.errors import DuploError

def main():
  try:
    duplo, args = DuploClient.from_env()
    o = duplo(*args)
    print(o)
  except DuploError as e:
    print(e)
    exit(e.code)

if __name__ == "__main__":
  main()



from duplocloud.client import DuploClient  # Importing necessary modules
from duplocloud.resource import DuploTenantResource
from duplocloud.commander import Command, Resource
import duplocloud.args as args  # Importing necessary modules

@Resource("job")  # Decorator to define a resource
class DuploJob(DuploTenantResource):
  
  def __init__(self, duplo: DuploClient):  # Constructor method
    super().__init__(duplo)  # Calling superclass constructor
  
  @Command()  # Decorator to define a command
  def list(self):  # Method to list all jobs
    """Retrieve a list of all jobs in a tenant."""
    tenant_id = self.tenant["TenantId"]  # Getting tenant ID
    response = self.duplo.get(f"v3/subscriptions/{tenant_id}/k8s/job")  # Making GET request
    return response.json()  # Returning JSON response
  
  @Command()  # Decorator to define a command
  def find(self, name: args.NAME):  # Method to find a job by name
    """Find a job by name.
    
    Args:
      name (str): The name of the job to find.
    Returns: 
      The job object.
    Raises:
      DuploError: If the job could not be found.
    """
    tenant_id = self.tenant["TenantId"]  # Getting tenant ID
    response = self.duplo.get(f"v3/subscriptions/{tenant_id}/k8s/job/{name}")  # Making GET request
    return response.json()  # Returning JSON response

  @Command()  # Decorator to define a command
  def create(self, body: args.BODY):  # Method to create a new job
    """Create a new job.
    
    Args:
      job_config (dict): The configuration for the new job.
    Returns: 
      A message indicating the successful creation of the job.
    """
    tenant_id = self.tenant["TenantId"]  # Getting tenant ID
    job_name = body.get('JobName', 'Unnamed Job')  # Getting job name from job configuration
    self.duplo.post(f"v3/subscriptions/{tenant_id}/k8s/job", body)  # Making POST request with job configuration
    return {
      "message": f"Job {job_name} created"  # Returning message with job name
    }



from duplocloud.client import DuploClient
from duplocloud.resource import DuploTenantResource
from duplocloud.errors import DuploError
from duplocloud.commander import Command, Resource
import duplocloud.args as args

@Resource("hosts")
class DuploHosts(DuploTenantResource):
  
  def __init__(self, duplo: DuploClient):
    super().__init__(duplo)
  
  @Command()
  def list(self):
    """Retrieve a list of all hosts in a tenant."""
    tenant_id = self.tenant["TenantId"]
    response = self.duplo.get(f"subscriptions/{tenant_id}/GetNativeHosts")
    return response.json()

  @Command()
  def find(self, 
           name: args.NAME):
    """Find a host by name.
    
    Args:
      name (str): The name of the host to find.
    Returns: 
      The host object.
    Raises:
      DuploError: If the host could not be found.
    """
    try:
        return [s for s in self.list() if s["FriendlyName"] == name][0]
    except IndexError:
      raise DuploError(f"Host '{name}' not found", 404)
